@@ -1,6 +1,6 @@
 /* Compare numeric strings.
 
-   Copyright (C) 2005-2013 Free Software Foundation, Inc.
+   Copyright (C) 2005-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Paul Eggert.  */
 
@@ -23,7 +23,7 @@
 
 /* Externally-visible name for numcompare.  */
 
-int _GL_ATTRIBUTE_PURE
+int
 strnumcmp (char const *a, char const *b,
            int decimal_point, int thousands_sep)
 {
