@@ -1,6 +1,5 @@
 /* touch -- change modification and access times of files
-   Copyright (C) 1987, 1989-1991, 1995-2005, 2007-2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1987-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Written by Paul Rubin, Arnold Robbins, Jim Kingdon, David MacKenzie,
    and Randy Smith. */
@@ -22,11 +21,10 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
-#include <assert.h>
 
 #include "system.h"
 #include "argmatch.h"
-#include "error.h"
+#include "assure.h"
 #include "fd-reopen.h"
 #include "parse-datetime.h"
 #include "posixtm.h"
@@ -35,7 +33,7 @@
 #include "stat-time.h"
 #include "utimens.h"
 
-/* The official name of this program (e.g., no `g' prefix).  */
+/* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "touch"
 
 #define AUTHORS \
@@ -45,7 +43,7 @@
   proper_name ("David MacKenzie"), \
   proper_name ("Randy Smith")
 
-/* Bitmasks for `change_times'. */
+/* Bitmasks for 'change_times'. */
 #define CH_ATIME 1
 #define CH_MTIME 2
 
@@ -83,46 +81,45 @@ enum
 
 static struct option const longopts[] =
 {
-  {"time", required_argument, NULL, TIME_OPTION},
-  {"no-create", no_argument, NULL, 'c'},
-  {"date", required_argument, NULL, 'd'},
-  {"reference", required_argument, NULL, 'r'},
-  {"no-dereference", no_argument, NULL, 'h'},
+  {"time", required_argument, nullptr, TIME_OPTION},
+  {"no-create", no_argument, nullptr, 'c'},
+  {"date", required_argument, nullptr, 'd'},
+  {"reference", required_argument, nullptr, 'r'},
+  {"no-dereference", no_argument, nullptr, 'h'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
-/* Valid arguments to the `--time' option. */
-static char const* const time_args[] =
+/* Valid arguments to the '--time' option. */
+static char const *const time_args[] =
 {
-  "atime", "access", "use", "mtime", "modify", NULL
+  "atime", "access", "use", "mtime", "modify", nullptr
 };
 
-/* The bits in `change_times' that those arguments set. */
+/* The bits in 'change_times' that those arguments set. */
 static int const time_masks[] =
 {
   CH_ATIME, CH_ATIME, CH_ATIME, CH_MTIME, CH_MTIME
 };
 
-/* Store into *RESULT the result of interpreting FLEX_DATE as a date,
-   relative to NOW.  If NOW is null, use the current time.  */
+/* The interpretation of FLEX_DATE as a date, relative to NOW.  */
 
-static void
-get_reldate (struct timespec *result,
-             char const *flex_date, struct timespec const *now)
+static struct timespec
+date_relative (char const *flex_date, struct timespec now)
 {
-  if (! parse_datetime (result, flex_date, now))
+  struct timespec result;
+  if (! parse_datetime (&result, flex_date, &now))
     error (EXIT_FAILURE, 0, _("invalid date format %s"), quote (flex_date));
+  return result;
 }
 
 /* Update the time of file FILE according to the options given.
    Return true if successful.  */
 
 static bool
-touch (const char *file)
+touch (char const *file)
 {
-  bool ok;
   int fd = -1;
   int open_errno = 0;
   struct timespec const *t = newtime;
@@ -132,17 +129,9 @@ touch (const char *file)
   else if (! (no_create || no_dereference))
     {
       /* Try to open FILE, creating it if necessary.  */
-      int default_permissions =
-        S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
       fd = fd_reopen (STDIN_FILENO, file,
-                      O_WRONLY | O_CREAT | O_NONBLOCK | O_NOCTTY,
-                      default_permissions);
-
-      /* Don't save a copy of errno if it's EISDIR, since that would lead
-         touch to give a bogus diagnostic for e.g., `touch /' (assuming
-         we don't own / or have write access to it).  On Solaris 5.6,
-         and probably other systems, it is EINVAL.  On SunOS4, it's EPERM.  */
-      if (fd == -1 && errno != EISDIR && errno != EINVAL && errno != EPERM)
+                      O_WRONLY | O_CREAT | O_NONBLOCK | O_NOCTTY, MODE_RW_UGO);
+      if (fd < 0)
         open_errno = errno;
     }
 
@@ -153,52 +142,62 @@ touch (const char *file)
         newtime[0].tv_nsec = UTIME_OMIT;
       else
         {
-          assert (change_times == CH_ATIME);
+          affirm (change_times == CH_ATIME);
           newtime[1].tv_nsec = UTIME_OMIT;
         }
     }
 
   if (amtime_now)
     {
-      /* Pass NULL to futimens so it will not fail if we have
+      /* Pass nullptr to futimens so it will not fail if we have
          write access to the file, but don't own it.  */
-      t = NULL;
+      t = nullptr;
     }
 
-  ok = (fdutimensat (fd, AT_FDCWD, (fd == STDOUT_FILENO ? NULL : file), t,
-                     (no_dereference && fd == -1) ? AT_SYMLINK_NOFOLLOW : 0)
-        == 0);
+  char const *file_opt = fd == STDOUT_FILENO ? nullptr : file;
+  int atflag = no_dereference ? AT_SYMLINK_NOFOLLOW : 0;
+  int utime_errno = (fdutimensat (fd, AT_FDCWD, file_opt, t, atflag) == 0
+                     ? 0 : errno);
 
   if (fd == STDIN_FILENO)
     {
       if (close (STDIN_FILENO) != 0)
         {
-          error (0, errno, _("closing %s"), quote (file));
+          error (0, errno, _("failed to close %s"), quoteaf (file));
           return false;
         }
     }
   else if (fd == STDOUT_FILENO)
     {
       /* Do not diagnose "touch -c - >&-".  */
-      if (!ok && errno == EBADF && no_create)
+      if (utime_errno == EBADF && no_create)
         return true;
     }
 
-  if (!ok)
+  if (utime_errno != 0)
     {
-      if (open_errno)
+      /* Don't diagnose with open_errno if FILE is a directory, as that
+         would give a bogus diagnostic for e.g., 'touch /' (assuming we
+         don't own / or have write access).  On Solaris 10 and probably
+         other systems, opening a directory like "." fails with EINVAL.
+         (On SunOS 4 it was EPERM but that's obsolete.)  */
+      struct stat st;
+      if (open_errno
+          && ! (open_errno == EISDIR
+                || (open_errno == EINVAL
+                    && stat (file, &st) == 0 && S_ISDIR (st.st_mode))))
         {
           /* The wording of this diagnostic should cover at least two cases:
              - the file does not exist, but the parent directory is unwritable
              - the file exists, but it isn't writable
              I think it's not worth trying to distinguish them.  */
-          error (0, open_errno, _("cannot touch %s"), quote (file));
+          error (0, open_errno, _("cannot touch %s"), quoteaf (file));
         }
       else
         {
-          if (no_create && errno == ENOENT)
+          if (no_create && utime_errno == ENOENT)
             return true;
-          error (0, errno, _("setting times of %s"), quote (file));
+          error (0, utime_errno, _("setting times of %s"), quoteaf (file));
         }
       return false;
     }
@@ -210,8 +209,7 @@ void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-             program_name);
+    emit_try_help ();
   else
     {
       printf (_("Usage: %s [OPTION]... FILE...\n"), program_name);
@@ -223,11 +221,10 @@ is supplied.\n\
 \n\
 A FILE argument string of - is handled specially and causes touch to\n\
 change the times of the file associated with standard output.\n\
-\n\
-"), stdout);
-      fputs (_("\
-Mandatory arguments to long options are mandatory for short options too.\n\
 "), stdout);
+
+      emit_mandatory_arg_note ();
+
       fputs (_("\
   -a                     change only the access time\n\
   -c, --no-create        do not create any files\n\
@@ -242,18 +239,17 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 "), stdout);
       fputs (_("\
   -r, --reference=FILE   use this file's times instead of current time\n\
-  -t STAMP               use [[CC]YY]MMDDhhmm[.ss] instead of current time\n\
-  --time=WORD            change the specified time:\n\
-                           WORD is access, atime, or use: equivalent to -a\n\
-                           WORD is modify or mtime: equivalent to -m\n\
+  -t [[CC]YY]MMDDhhmm[.ss]  use specified time instead of current time,\n\
+                         with a date-time format that differs from -d's\n\
+"), stdout);
+      fputs (_("\
+      --time=WORD        specify which time to change:\n\
+                           access time (-a): 'access', 'atime', 'use';\n\
+                           modification time (-m): 'modify', 'mtime'\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      fputs (_("\
-\n\
-Note that the -d and -t options accept different time-date formats.\n\
-"), stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
@@ -264,7 +260,7 @@ main (int argc, char **argv)
   int c;
   bool date_set = false;
   bool ok = true;
-  char const *flex_date = NULL;
+  char const *flex_date = nullptr;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -277,7 +273,7 @@ main (int argc, char **argv)
   change_times = 0;
   no_create = use_ref = false;
 
-  while ((c = getopt_long (argc, argv, "acd:fhmr:t:", longopts, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, "acd:fhmr:t:", longopts, nullptr)) != -1)
     {
       switch (c)
         {
@@ -350,26 +346,24 @@ main (int argc, char **argv)
       if (no_dereference ? lstat (ref_file, &ref_stats)
           : stat (ref_file, &ref_stats))
         error (EXIT_FAILURE, errno,
-               _("failed to get attributes of %s"), quote (ref_file));
+               _("failed to get attributes of %s"), quoteaf (ref_file));
       newtime[0] = get_stat_atime (&ref_stats);
       newtime[1] = get_stat_mtime (&ref_stats);
       date_set = true;
       if (flex_date)
         {
           if (change_times & CH_ATIME)
-            get_reldate (&newtime[0], flex_date, &newtime[0]);
+            newtime[0] = date_relative (flex_date, newtime[0]);
           if (change_times & CH_MTIME)
-            get_reldate (&newtime[1], flex_date, &newtime[1]);
+            newtime[1] = date_relative (flex_date, newtime[1]);
         }
     }
   else
     {
       if (flex_date)
         {
-          struct timespec now;
-          gettime (&now);
-          get_reldate (&newtime[0], flex_date, &now);
-          newtime[1] = newtime[0];
+          struct timespec now = current_timespec ();
+          newtime[1] = newtime[0] = date_relative (flex_date, now);
           date_set = true;
 
           /* If neither -a nor -m is specified, treat "-d now" as if
@@ -379,12 +373,12 @@ main (int argc, char **argv)
               && newtime[0].tv_sec == now.tv_sec
               && newtime[0].tv_nsec == now.tv_nsec)
             {
-              /* Check that it really was "-d now", and not a time
-                 stamp that just happens to be the current time.  */
+              /* Check that it really was "-d now", and not a timestamp
+                 that just happens to be the current time.  */
               struct timespec notnow, notnow1;
               notnow.tv_sec = now.tv_sec ^ 1;
               notnow.tv_nsec = now.tv_nsec;
-              get_reldate (&notnow1, flex_date, &notnow);
+              notnow1 = date_relative (flex_date, notnow);
               if (notnow1.tv_sec == notnow.tv_sec
                   && notnow1.tv_nsec == notnow.tv_nsec)
                 date_set = false;
@@ -392,7 +386,7 @@ main (int argc, char **argv)
         }
     }
 
-  /* The obsolete `MMDDhhmm[YY]' form is valid IFF there are
+  /* The obsolete 'MMDDhhmm[YY]' form is valid IFF there are
      two or more non-option arguments.  */
   if (!date_set && 2 <= argc - optind && posix2_version () < 200112
       && posixtime (&newtime[0].tv_sec, argv[optind],
@@ -407,13 +401,13 @@ main (int argc, char **argv)
           struct tm const *tm = localtime (&newtime[0].tv_sec);
 
           /* Technically, it appears that even a deliberate attempt to cause
-             the above localtime to return NULL will always fail because our
+             the above localtime to return nullptr will always fail because our
              posixtime implementation rejects all dates for which localtime
              would fail.  However, skip the warning if it ever fails.  */
           if (tm)
             error (0, 0,
-                   _("warning: `touch %s' is obsolete; use "
-                     "`touch -t %04ld%02d%02d%02d%02d.%02d'"),
+                   _("warning: 'touch %s' is obsolete; use "
+                     "'touch -t %04ld%02d%02d%02d%02d.%02d'"),
                    argv[optind],
                    tm->tm_year + 1900L, tm->tm_mon + 1, tm->tm_mday,
                    tm->tm_hour, tm->tm_min, tm->tm_sec);
@@ -439,5 +433,5 @@ main (int argc, char **argv)
   for (; optind < argc; ++optind)
     ok &= touch (argv[optind]);
 
-  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
+  return ok ? EXIT_SUCCESS : EXIT_FAILURE;
 }
