@@ -1,6 +1,6 @@
 /* uname -- print system information
 
-   Copyright (C) 1989-2023 Free Software Foundation, Inc.
+   Copyright (C) 1989-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,8 +45,6 @@
 #endif
 
 #include "system.h"
-#include "die.h"
-#include "error.h"
 #include "quote.h"
 #include "uname.h"
 
@@ -83,27 +81,27 @@
 
 static struct option const uname_long_options[] =
 {
-  {"all", no_argument, NULL, 'a'},
-  {"kernel-name", no_argument, NULL, 's'},
-  {"sysname", no_argument, NULL, 's'},	/* Obsolescent.  */
-  {"nodename", no_argument, NULL, 'n'},
-  {"kernel-release", no_argument, NULL, 'r'},
-  {"release", no_argument, NULL, 'r'},  /* Obsolescent.  */
-  {"kernel-version", no_argument, NULL, 'v'},
-  {"machine", no_argument, NULL, 'm'},
-  {"processor", no_argument, NULL, 'p'},
-  {"hardware-platform", no_argument, NULL, 'i'},
-  {"operating-system", no_argument, NULL, 'o'},
+  {"all", no_argument, nullptr, 'a'},
+  {"kernel-name", no_argument, nullptr, 's'},
+  {"sysname", no_argument, nullptr, 's'},	/* Obsolescent.  */
+  {"nodename", no_argument, nullptr, 'n'},
+  {"kernel-release", no_argument, nullptr, 'r'},
+  {"release", no_argument, nullptr, 'r'},  /* Obsolescent.  */
+  {"kernel-version", no_argument, nullptr, 'v'},
+  {"machine", no_argument, nullptr, 'm'},
+  {"processor", no_argument, nullptr, 'p'},
+  {"hardware-platform", no_argument, nullptr, 'i'},
+  {"operating-system", no_argument, nullptr, 'o'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 static struct option const arch_long_options[] =
 {
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {NULL, 0, NULL, 0}
+  {nullptr, 0, nullptr, 0}
 };
 
 void
@@ -193,7 +191,8 @@ decode_switches (int argc, char **argv)
   if (uname_mode == UNAME_ARCH)
     {
       while ((c = getopt_long (argc, argv, "",
-                               arch_long_options, NULL)) != -1)
+                               arch_long_options, nullptr))
+             != -1)
         {
           switch (c)
             {
@@ -210,7 +209,8 @@ decode_switches (int argc, char **argv)
   else
     {
       while ((c = getopt_long (argc, argv, "asnrvmpio",
-                               uname_long_options, NULL)) != -1)
+                               uname_long_options, nullptr))
+             != -1)
         {
           switch (c)
             {
@@ -297,7 +297,7 @@ main (int argc, char **argv)
       struct utsname name;
 
       if (uname (&name) == -1)
-        die (EXIT_FAILURE, errno, _("cannot get system name"));
+        error (EXIT_FAILURE, errno, _("cannot get system name"));
 
       if (toprint & PRINT_KERNEL_NAME)
         print_element_env (name.sysname, "UNAME_SYSNAME");
